function [Vars err] = sffs(Data, Classes, repeat, validsize, testsize, K, max_secs)

%
% sequential forward float selection
%

[n dim]    = size(Data);
Cost       = NaN * zeros(dim, 1);
X          = cell(dim, 1);
err        = 1;
Vars       = [];
start_time = cputime;

%
% initialization (select the first two variables using sequential
% forward selection)
%

[Inds Costs] = sfs(Data, Classes, [], repeat, validsize, testsize, K, 2); 
X{1}         = Inds(1); X{2} = Inds;
Cost(1:2)    = Costs;
i            = 2;
Prev         = [];

if (Cost(1) < Cost(2))
  err  = Cost(1);
  Vars = X{1};
else
  err  = Cost(2);
  Vars = X{2};
end

%
% main loop
%

while((cputime - start_time) < max_secs & length(X{i} < dim))

  if (length(Prev) >= length(X{i})) & (sum(Prev(1:i)~=X{i})==0) % test for an infinite loop
    return;
  else
    Prev = X{i};
  end
  
  disp(Prev')
  disp(X{i}')
  
  if (Cost(i) < err)
    err  = Cost(i);
    Vars = X{i};
  end

  fprintf(1, '\ri = %d, current = %.4f, best = %.4f, group: ', i, ...
          Cost(i), err);
  fprintf(1, '%d ', Vars);
  fprintf(1, '\n');
  
  %
  % step 1: inclusion using sequential forward selection
  %

  [ind_inc cost_inc] = sfs(Data, Classes, X{i}, repeat, validsize, testsize, K, 1);

  XX   = [X{i}; ind_inc];
  
  %
  % step 2: conditional exclusion
  % (we actually use sbs routine to determine the candidate)
  %
  
  [dummy1 dummy2 ind_excl cost_excl] = ...
    sbs(Data, Classes, XX, repeat, validsize, testsize, K, 1);
  
  if (ind_excl == ind_inc) 

    i           = i + 1;
    X{i}        = XX;
    Cost(i)     = cost_inc;
  
  else

    XX(XX == ind_excl) = [];
    cost_xx = cost_excl;

    % step 3: continuation of conditional exclusion
    % (again, the sbs routine is used)
      
    while (i > 2)

      [dummy1 dummy2 ind_excl cost_xx_excl] = ...
         sbs(Data, Classes, XX, repeat, validsize, testsize, K, 1);

      if (cost_xx_excl > Cost(i - 1))
        break; 
      else
        XX(XX == ind_excl) = [];
        cost_xx            = cost_xx_excl; 
        i                  = i - 1;
      end

    end
  
    X{i}    = XX;
    Cost(i) = cost_xx;

  end


end

