%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% This m-file includes Matlab code for the synthetic data experiment
% reported in article "Compact Modeling of Data Using Independent
% Variable Group Analysis" by Esa Alhoniemi, Antti Honkela, Krista
% Lagus, Jeremias Sepp, Paul Wagner, and Harri Valpola
%

addpath /share/ivga/releases/ivga-1.14.5/program % add your own ivga path here

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Generate dataset with 1000 points
% (variables: (1) education, (2) income, (3) height, (4) weight)

Data = zeros(1000, 4);

for i = 1:1000
 
  % education and income are mutually dependent,
  % but independent on weight and height

  switch(ceil(4 * rand))
   case 1
    M = [14 30]; S = [0.92 2.3];
   case 2
    M = [18 36]; S = [0.92 3.22];
   case 3
    M = [18 27]; S = [0.92 2.76];
   case 4
    M = [22 34]; S = [0.92 2.3];
  end 

  Data(i, 1:2) = M + S .* randn(1,2); 
  
  % weight and height are mutually dependent,
  % but independent on education and income
  
  Temp         = 38  + 9.2 * randn;
  Data(i, 4)   = Temp + 40;
  Data(i, 3)   = Temp + 140 + 0.161 * (Temp + 5) * randn;
  
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% run ivga 100 times and display the results
%

Varnames = 'eihw';

for j = 1:100

  result  = ivga(ivga_import_data(Data), 'quiet', 'runtime', 'slower');

  % determine correct order for the variables
  
  ngroups    = size(result.group_models, 2);
  First_vars = [];
  for k = 1:ngroups
    Vars       = sort(result.group_models(k).member_variables);
    First_vars = [First_vars Vars(1)];
  end
  [Dummy Order] = sort(First_vars);

  % display groups & compute no. of parameters
  
  params  = 0;
  for l = Order

    Vars          = result.group_models(l).member_variables;
    nvars         = length(Vars);
    ncentroids    = get_ncentroids(result.group_models(l).internal);
    params        = params + nvars * (6 + 4 * ncentroids);

    fprintf('{');
    for m = 1:nvars 
      fprintf('%s', Varnames(Vars(m)));
      if (m < nvars)
        fprintf(', ');
      end
    end
    fprintf('}');
  end

  fprintf(' %.1f %d\n', result.cost, params); % display cost &
                                              % no. of parameters
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
